@@ -5,7 +5,7 @@ import time
 
 
 __author__ = 'Daniel Lindsley'
-__version__ = (0, 2, 0)
+__version__ = (0, 3, 0)
 __license__ = 'BSD'
 
 
@@ -36,15 +36,19 @@ class Pool(object):
     Optionally accepts a ``workers`` kwarg. Default is 1.
     
     Optionally accepts a ``debug`` kwarg. Default is False.
+    
+    Optionally accepts a ``wait_time`` kwarg. Default is 0.1.
     """
-    def __init__(self, workers=1, debug=False):
+    def __init__(self, workers=1, debug=False, wait_time=0.1):
         if workers < 1:
             raise NotEnoughWorkers("You need to use at least one worker.")
         
         self.workers = workers
         self.pool = {}
         self.commands = []
+        self.callback = None
         self.debug = debug
+        self.wait_time = wait_time
     
     def prepare_commands(self, commands):
         """
@@ -79,13 +83,36 @@ class Pool(object):
         
         return None
     
+    def process_kwargs(self, command):
+        """
+        A hook to alter the kwargs given to ``subprocess.Process``.
+
+        Takes a ``command`` argument, which is unused by default, but can be
+        used to switch the flags used.
+
+        By default, only specifies ``shell=True``.
+        """
+        return {
+            'shell': True,
+        }
+    
     def create_process(self, command):
         """
         Given a provided command (string or list), creates a new process
         to execute the command.
         """
         logging.debug("Starting process to handle command '%s'." % command)
-        return subprocess.Popen(command, shell=True)
+        kwargs = self.process_kwargs(command)
+        return subprocess.Popen(command, **kwargs)
+    
+    def set_callback(self, callback=None):
+        """
+        Sets up a callback to be run whenever a process finishes.
+
+        If called with ``None`` or without any args, it will clear any
+        existing callback.
+        """
+        self.callback = callback
     
     def add_to_pool(self, proc):
         """
@@ -112,9 +139,13 @@ class Pool(object):
         """
         A hook for inspecting the pool's current status.
         
-        By default, simply makes a log message.
+        By default, simply makes a log message and returns the length of
+        the pool.
         """
-        logging.debug("Current pool size: %s" % len(self.pool))
+        # Call ``len()`` just once.
+        pool_size = len(self.pool)
+        logging.debug("Current pool size: %s" % pool_size)
+        return pool_size
     
     def busy_wait(self):
         """
@@ -122,9 +153,9 @@ class Pool(object):
         
         By default, sleeps for 0.1 seconds.
         """
-        time.sleep(0.1)
+        time.sleep(self.wait_time)
     
-    def run(self, commands=None):
+    def run(self, commands=None, callback=None):
         """
         The method to actually execute all the commands with the pool.
         
@@ -133,6 +164,9 @@ class Pool(object):
         """
         if commands is not None:
             self.prepare_commands(commands)
+
+        if callback is not None:
+            self.set_callback(callback)
         
         keep_running = True
         
@@ -154,6 +188,9 @@ class Pool(object):
                 logging.debug("Checking status on %s" % self.pool[pid].pid)
                 
                 if self.pool[pid].poll() >= 0:
+                    if self.callback:
+                        self.callback(self.pool[pid])
+
                     self.remove_from_pool(pid)
             
             keep_running = self.command_count() or len(self.pool) > 0
